class Stepper {
  private:
  int pin1shift, pin2shift, pin3shift, pin4shift;
  volatile byte& reg1;
  volatile byte& reg2;
  volatile byte& reg3;
  volatile byte& reg4;
  int stepPos;
  int delayTime;

  public:
  Stepper(volatile byte& r1, int p1, volatile byte& r2, int p2, volatile byte& r3, int p3, volatile byte& r4, int p4, int del): pin1shift(p1), reg1(r1), pin2shift(p2), reg2(r2), pin3shift(p3), reg3(r3), pin4shift(p4), reg4(r4), delayTime(del) {
    stepPos = 0;
  }

  private:
  void out1() {
    reg1 |= 1 << pin1shift;
  }

  void clear1() {
    reg1 &= ~(1 << pin1shift);
  }

  void out2() {
    reg2 |= 1 << pin2shift;
  }

  void clear2() {
    reg2 &= ~(1 << pin2shift);
  }

  void out3() {
    reg3 |= 1 << pin3shift;
  }

  void clear3() {
    reg3 &= ~(1 << pin3shift);
  }

  void out4() {
    reg4 |= 1 << pin4shift;
  }

  void clear4() {
    reg4 &= ~(1 << pin4shift);
  }

  void runStep() {
    switch (stepPos) {
      case 0:
      out1();
      clear2();
      out3();
      clear4();
      break;
      case 1:
      clear1();
      out2();
      out3();
      clear4();
      break;
      case 2:
      clear1();
      out2();
      clear3();
      out4();
      break;
      case 3:
      out1();
      clear2();
      clear3();
      out4();
      break;
    }
    delay(delayTime);
  }

  public:
  void stepCW(int numSteps) {
    for (int i = 0; i < numSteps; ++i) {
      stepPos = (stepPos + 1) % 4;
      runStep();
    }
  }

  void stepCCW(int numSteps) {
    for (int i = 0; i < numSteps; ++i) {
      --stepPos;
      if (stepPos < 0) stepPos = 3;
      runStep();
    }
  }
};

void setup() {
  // put your setup code here, to run once:
  PORTA.DIRSET |= 1 << 4;
  PORTA.DIRSET |= 1 << 5;
  PORTA.DIRSET |= 1 << 3;
  PORTB.DIRSET |= 1 << 2;
}

Stepper myStepper(PORTA.OUT, 4, PORTA.OUT, 5, PORTB.OUT, 2, PORTA.OUT, 3, 10);

void loop() {
  // put your main code here, to run repeatedly:
  myStepper.stepCW(200);
  delay(1000);
  myStepper.stepCCW(200);
  delay(1000);
}
